/* DOM elements to cache, to keep as much work out of the scroll handler as possible */
var	gameHeaderWrapper = null,
	gameHeaderTransition = null,
	placeholder = null,
	gameContainer = null,
	updatePlaceholderHeightOnScroll = false;
/* Cached properties of elements */
var	currScrollState = 1, 
	newScrollState = 1,
	pageScroll = 0, 
	maxScroll = 0, 
	ghbOriginalTop = 0,
	gcHeight = 0,
	gcTop = 0,
	gcBottom = 0,
	ghbHeight = 0,
	scrollPadding = 0,
	placeholderHeight;
/* Flags to determine if functionality is supported */
var mobileDevice = false,
	unsupportedBrowser = false;

function updatePlaceholderHeight() {
	var gameHeaderWrapperInnerHeight = gameHeaderWrapper.innerHeight();
	if(gameHeaderWrapperInnerHeight != placeholderHeight) {
		placeholderHeight = gameHeaderWrapperInnerHeight;
		placeholder.css({height: placeholderHeight +"px"});
	}
}

function scrollHandler() {
	/* Calculate these here to support games where the container resizes during gameplay */
	if (gcHeight != gameContainer.height()) {
		gcHeight = gameContainer.height();
		gcBottom = gcTop + gcHeight;
		maxScroll = gcBottom - scrollPadding < 0 ? 0 : gcBottom - scrollPadding;
	}
	
	pageScroll = parseInt(this.pageYOffset);
	if (pageScroll >= ghbOriginalTop - 1) {
		if (pageScroll > maxScroll) {
			newScrollState = 3;
		} else {
			newScrollState = 2;
		}
	} else {
		newScrollState = 1;
	}

	if(updatePlaceholderHeightOnScroll) {
		updatePlaceholderHeight();
	}

	if (newScrollState !== currScrollState || newScrollState === 3) {
		if (newScrollState == 1) {
			gameHeaderWrapper.removeClass('fixed-header').css({top:''});
			gameHeaderTransition.hide();
			placeholder.hide();
		} else if (newScrollState == 2) {
			gameHeaderWrapper.addClass('fixed-header').css({top:''});
			gameHeaderTransition.show();
			placeholder.show();
		} else if (newScrollState == 3) {
			gameHeaderWrapper.addClass('fixed-header').css({top:(maxScroll - pageScroll)+"px"});
			gameHeaderTransition.show();
			placeholder.show();
		}
		currScrollState = newScrollState;
	}
}

function resetScrollHandler() {
	$j(window).off('scroll.fixedHeader');
	
	if (!mobileDevice && !unsupportedBrowser) {
		/* Calculate these here to support clickable games where the container resizes after you click PLAY */
		if (gcHeight != gameContainer.height()) {
			gcHeight = gameContainer.height();
			gcBottom = gcTop + gcHeight; 
			maxScroll = gcBottom - scrollPadding < 0 ? 0 : gcBottom - scrollPadding;
		}
		
		if (gcHeight + ghbHeight > window.innerHeight && window.innerHeight > scrollPadding) {
			$j(window).on('scroll.fixedHeader', scrollHandler);
		} else {
			if (currScrollState > 1) {
				/* Reset everything back to normal because the window is big enough to show the whole game */
				gameHeaderWrapper.removeClass('fixed-header').css({top:"0px"});
				gameHeaderTransition.hide();
				placeholder.hide();
				currScrollState = 1;
			}
		}
	}
}

$j(function () {
	var MutationObserver = window.MutationObserver || window.WebKitMutationObserver,
		observer;
	mobileDevice = (/iphone|ipad|ipod|android|blackberry|mini|windows\sce|palm/i.test(navigator.userAgent.toLowerCase()));
	unsupportedBrowser = (typeof window.pageYOffset === 'undefined' || typeof window.innerHeight === 'undefined');

	if (!mobileDevice && !unsupportedBrowser && typeof _spHooks !== 'undefined') {
		placeholder = $j('#placeholder');
		gameHeaderWrapper = $j('#gameHeaderWrapper');
		gameHeaderTransition = $j('#gameHeaderTransition');
		
		if ($j('#gameTable').length) {
			gameContainer = $j('#gameTable');
		} else if ($j('#mapcanvas').length) {
			gameContainer = $j('#mapcanvas');
		} else if ($j('#answerbox').length) {
			gameContainer = $j('#answerbox');
		} else if ($j('#grid-container').length) {
			gameContainer = $j('#grid-container');
		}
		
		if (!gameContainer || !gameHeaderWrapper || !gameHeaderTransition || !placeholder) {
			/* Something is missing on the page, bail. */
			return;
		}

		updatePlaceholderHeight();  // initial placeholder height
		if(MutationObserver) {  // use mutation observer so placeholder height is updated when question changes.
			observer = new MutationObserver(function (mutations, observer) {
				updatePlaceholderHeight();
			});
			observer.observe(gameHeaderWrapper[0], {
				childList: true,
				subtree: true
			});
		} else {  // no dom observer (Most likely IE browser before IE11 or Opera Mini). Update when they next scroll.
			updatePlaceholderHeightOnScroll = true;
		}

		gcTop = gameContainer.offset().top;
		ghbOriginalTop = gameHeaderWrapper.offset().top;
		ghbHeight = gameHeaderWrapper.height();
		scrollPadding = ghbHeight * 2;
		
		if (gcTop == 0 && $j('#preHide').length) {
			gcTop = $j('#preHide').offset().top + parseInt(gameContainer.css('marginTop'));
		}
					
		$j(window).on('resize', function() {
			scrollHandler();
			resetScrollHandler(); 
		});
		
		_spHooks['start'].push(resetScrollHandler);

		_spHooks['end'].push(function() {
			$j(window).off('scroll.fixedHeader');
			gameHeaderWrapper.removeClass('fixed-header').css({top:"0px"});
			gameHeaderTransition.hide();
			placeholder.hide();
		});
	}
});
