window.SporcleLib = window.SporcleLib || {};
window.SporcleLib.Modal = window.SporcleLib.Modal || {};

SporcleLib.Modal._openModal = function(html, args, callback, templateVars){
	// Store our lastClickedEl so we can return to it after the user exits a Modal.
	// Stored in the Modal object as const/let/var variables instances don't change in event listener after first assignment even when in a parent scope.
	if (SporcleLib.Modal.lastClickedEl === undefined) {
		SporcleLib.Modal.lastClickedEl = document.activeElement;
	}

    if (typeof SporcleLib.Modal._remodal === 'undefined') {
        SporcleLib.Modal._remodal = $j(document.createElement('div')).remodal();
	}

	SporcleLib.Modal._remodal.$modal.on('opened', function () {
		$j('input').not('.remodal input, input[data-no-tabindex]').attr('tabindex', "-1");
		
		// Initializes the first trap of modal focus.
		trapModalFocus();
	});

	SporcleLib.Modal._remodal.$modal.on('closed', function () {
		SporcleLib.Modal._remodal.$modal.attr({class: 'remodal remodal-is-initialized remodal-is-closed'});
		SporcleLib.Modal._remodal.$overlay.attr({class: 'remodal-overlay remodal-is-closed'});

		if(args.hasOwnProperty('onClose')){
			args.onClose(SporcleLib.Modal._remodal);
		}

		reorderTabIndex(0);

		returnFocusFromModal();
	});

    if (typeof args !== 'undefined' && typeof args.switching !== 'undefined' && args.switching) {
        var content = typeof templateVars !== 'undefined' ? _.template(html)(templateVars) : html;

        SporcleLib.Modal._remodal.switch($j(document.createElement('div')), args, content);

        if (typeof callback === 'function') {
            callback(SporcleLib.Modal._remodal);
        }

        SporcleLib.Modal._started = undefined;

    } else {
        if (typeof templateVars !== 'undefined') {
            SporcleLib.Modal._remodal.$modal.html(_.template(html)(templateVars));
        } else {
            SporcleLib.Modal._remodal.$modal.html(html);
        }

        if (typeof callback === 'function') {
            callback(SporcleLib.Modal._remodal);
        }

        SporcleLib.Modal._remodal.open();

        SporcleLib.Modal._started = undefined;
    }

	// This call is incase there this is a second modal popup in the UI such as the Register Modal. Otherwise the focus would default to the tabbable element list for the initial modal and would fail to work correctly.
	trapModalFocus();
};

SporcleLib.Modal._openModalFromAjax = function(url, args, callback, templateVars, modalParams) {
    if (typeof SporcleLib.Modal._started === 'undefined') {
		SporcleLib.Modal._started = 'opening';

		modalParams = typeof modalParams !== 'undefined' ? modalParams : {};

		var ajaxMethod = 'GET';
		if (typeof modalParams.ajaxMethod !== 'undefined' && modalParams.ajaxMethod && ['GET', 'POST'].indexOf(modalParams.ajaxMethod) > -1) {
			ajaxMethod = modalParams.ajaxMethod;
		}

		$j.ajax({
			type : ajaxMethod,
			data : args,
			url  : url
		})
		.done(function(response) {
			SporcleLib.Modal._openModal(response, args, callback, templateVars)
		})
        .fail(function (xhr)  {
            var message;

            if (xhr.responseJSON && xhr.responseJSON.message) {
                message = xhr.responseJSON.message;
            } else if (xhr.hasOwnProperty('responseText') && xhr.responseText){
                var res = JSON.parse(xhr.responseText);
                if(res.hasOwnProperty('message') && res.message){
                    message = res.message;
                }
            }

            if(!message){
                message = 'Oops, something went wrong when creating your challenge. Please refresh and try again.';
            }

            showMainPageAlert(message, {
                type: 'danger',
                dismissible: true
            });

            SporcleLib.Modal._started = undefined;
            SporcleLib.Modal._remodal = undefined;
		});
	}
};

SporcleLib.Modal.openLoginModal = function(args) {
	var payload = window._payload || {} ;
	var signinAction = payload.register_action = (typeof args.signinAction != 'undefined' ? args.signinAction : function () {return function () {window.location.reload();};});
	window._payload = payload;

	SporcleLib.Modal._openModalFromAjax('/ajax/modals/login.php', args, function(remodal) {
		activateFloatingLabels();
		SporcleLib.Modal._remodal.$modal.attr({class:'remodal remodal-is-initialized remodal-is-opened registration'});

		var $handle = remodal.$modal.find('#email');
		var $rememberMe = remodal.$modal.find('#remember-me');
		var $password = remodal.$modal.find('#password');
		
		var $loginButton = remodal.$modal.find('#log-in-button');
		var $loginButtonText = remodal.$modal.find('#login-text');
		var $loginButtonSpinner = remodal.$modal.find('#login-spinner');
		var $facebookButton = remodal.$modal.find('.facebook-signup-button');
		var $googleButton = remodal.$modal.find('.google-signup-button');
		var $switchButton = remodal.$modal.find('#switch-to-join-modal');
		var $closeButton = remodal.$modal.find('.closeButton');
		var inProgress = false;
		
		var $errors = remodal.$modal.find('.registration-error');

		var loginFacebook = window.fbLogin;
		var tryLogin = window.attemptLogin;
		$j('.initGoogleLogin').each(function() {
			$googleButton = $j(this);

			$googleButton.on('click', function() {
				gLogin(args.regPath, signinAction, null, null);
			});
		});


		if (!window.auth2Initialized && typeof initAuth2 === 'function') {
			initAuth2();
		}
		
		const clearErrors = function() {
			$errors.hide().empty();
		}

		var doLogin = function() {
			if (inProgress) {
				return;
			}
			inProgress = true;
			
		    ga('send', 'event', 'Login Modal', args.analyticAction + ' - Log In', window.location.href);
			clearErrors();
			$loginButtonText.hide();
			$loginButtonSpinner.show();
		 
			tryLogin($handle.val(), $password.val(), $rememberMe.is(':checked'), function(data) {
				if (data.success === true && data.logged_in === true) {
					payload.register_action()();
				} else {
					$loginButtonSpinner.hide();
					const errors = data.errors || [];
					errors.forEach(function(error) {
						$j('#'+error.field+'-error').text(error.message).show();
					});
					$loginButtonText.show();
					inProgress = false;
				}
			});
		};

		var handleKeyPress = function(e) {
			clearErrors();
			if (e.which === 13 || e.keyCode === 13) {
				doLogin();
			}
		};

		$loginButton.on('click', doLogin);

		$handle.on('keyup', handleKeyPress);
		$password.on('keyup', handleKeyPress);

		$facebookButton.on('click', function() {
			loginFacebook('/facebook/endpoints/connect_fb.php', false /*isMobile*/, signinAction, args.regPath, window.location.href, '', $facebookButton, $j("<div class=\"ajax-loader\"></div>"));
			ga('send', 'event', 'Login Modal', args.analyticAction + ' - Connect with Facebook', window.location.href);
		});

		$googleButton.on('click', function() {
			ga('send', 'event', 'Login Modal', args.analyticAction + ' - Connect with Google', window.location.href);
		});

		$switchButton.on('click', function () {
            ga('send', 'event', 'Login Modal', args.analyticAction + ' - Join for Free', window.location.href);
			window.SporcleLib.Modal.openRegisterModal({
		        switching      : 1,
				analyticAction : args.analyticAction + ' - Join for Free',
				regPath		   : args.regPath,
				registerAction : signinAction
			});
        });

		$closeButton.on('click', function () {
           ga('send', 'event', 'Login Modal', args.analyticAction + ' - Close modal', window.location.href);
        });

		if (args.switching) {
			$handle.trigger('focus');

			remodal.$modal.one('closed', returnFocusFromModal);
		} else {
			remodal.$modal.on('opened', function() {
				$handle.trigger('focus');
			});
		}
	});

	ga('send', 'event', 'Login Modal', args.analyticAction, window.location.href);
};

SporcleLib.Modal.openUnverifiedModal = function(args) {
	var payload = window._payload || {} ;
	payload.register_action = (typeof args.registerAction != 'undefined' ? args.registerAction : function() {return function() {window.location.reload();}});
	window._payload = payload;

	SporcleLib.Modal._openModalFromAjax('/ajax/modals/unverified_promo.php', args, function(remodal) {
		remodal.$modal.attr('id', 'verifyPromo');

		remodal.$modal.find('#verifyPromoResendEmail').on('click', function(e) {
			var $this = $j(this);
			var $verifyText = remodal.$modal.find('#verifyPromoText');

			$j.ajax({
				type : 'GET',
				url  : '/ajax/banners.php',
				data : {
					banner : 'banner-verify',
					resend : 1
				}
			})
			.done(function(response) {
				var $el = $j(document.createElement('span'));
				if (response.success) {
					$el.html('Resent your verification e-mail. Check your e-mail inbox and click the link we sent you.');
				} else {
					$el.html('Too many resend attempts. Try again later.');
				}

				$verifyText.html($el);
				$this.remove();
			});

			ga('send', 'event', 'Unverified Modal', 'Resend Email', window.location.href);
		});
	});
};

SporcleLib.Modal.openReverifyModal = function(args) {
	var payload = window._payload || {} ;
	// payload.register_action = (typeof args.registerAction != 'undefined' ? args.registerAction : function() {return function() {window.location.reload();}});
	window._payload = payload;

	SporcleLib.Modal._openModalFromAjax('/ajax/modals/reverify.php', args, function(remodal) {
		remodal.settings.closeOnOutsideClick = false;
		remodal.settings.closeOnEscape = false;

		SporcleLib.Modal._remodal.$modal.attr({class:'remodal remodal-is-initialized remodal-is-opened registration'});
		remodal.$modal.attr('id', 'reverify');

		$j('#modal-submit-verify-code').on('click', function() {

		    var email = $j('#modal-user-email').length ? $j('#modal-user-email').text() : $j('#modal-new-email').val();

		    $j.ajax({
		        method : 'post',
		        url    : '/auth/ajax/checkemail.php',
		        data   : {
		            email         : email,
		            existing_user : true
		        },
		    }).done(function(data, status, xhr) {
		        if (data.success == true) {
					args.switching = 1;
					args.stage = 'verification';
					args.reverify = true;
		            window.SporcleLib.Modal.openRegisterModal(args);
		        } else {
		            if (data.message) {
		                $j('#modal-email-error').html(data.message);
		                $j('#modal-new-email').addClass('error');
		                $j('#modal-email-error').show();
		                $j('.resend-link').on('click', resendCode);
		            } else {
		                $j('#modal-email-error').html('There was an error, please try again.');
		                $j('#modal-email-error').show();
		            }
		        }
		    }).fail(function(data) {
		        if (data.responseJSON.message) {
		            $j('#modal-email-error').html(data.responseJSON.message);
		        } else {
		            $j('#modal-email-error').html('There was an error, please try again.');
		        }
		        $j('#modal-email-error').show();
		    });
		});
	});
};

SporcleLib.Modal.openRenameModal = function(args) {
	var payload = window._payload || {} ;
	window._payload = payload;

	SporcleLib.Modal._openModalFromAjax('/ajax/modals/rename.php', args, function(remodal) {
		SporcleLib.Modal._remodal.$modal.attr({class:'remodal remodal-is-initialized remodal-is-opened verify'});
	});
};

SporcleLib.Modal.openRegisterModal = function(args) {
	var payload = window._payload || {} ;
	payload.register_action = (typeof args.registerAction != 'undefined' && args.registerAction ? args.registerAction : function() {return function() {window.location.reload();};});
	window._payload = payload;

	args = args || {};

	if (typeof args.userKey === 'undefined' && typeof payload.new_user !== 'undefined'){
	    args.userKey = typeof payload.new_user.user_key !== 'undefined' ? payload.new_user.user_key : '';
    }

	SporcleLib.Modal._openModalFromAjax('/ajax/modals/register.php', args, function(remodal) {
		activateFloatingLabels();
		SporcleLib.Modal._remodal.$modal.attr({class:'remodal remodal-is-initialized remodal-is-opened registration'});

		if (args.switching) {
            if ($j('#email').is(':disabled')) {
                $j('#password1').trigger('focus');
            } else {
                $j('#email').trigger('focus');
            }
            $j('#modal-code-letter-1').trigger('focus');

            remodal.$modal.one('closed', returnFocusFromModal);
        } else {
            remodal.$modal.on('opened', function () {
                $j('#email').trigger('focus');
                if (window._payload.reverify) {
                    remodal.settings.closeOnOutsideClick = false;
                    remodal.settings.closeOnEscape = false;
                }
            });
        }

		if (typeof args.stage !== 'undefined' && (args.stage === 'promo' || args.stage === 'success')) {
			SporcleLib.Modal._remodal.$modal.attr({class: 'remodal remodal-is-initialized remodal-is-opened registration '+args.stage});
			remodal.$modal.one('closed', function () {
				payload.register_action()();
			});
		}
	});
};

SporcleLib.Modal.openVerifyModal = function(args) {
	var payload = window._payload || {} ;
	payload.register_action = (typeof args.registerAction != 'undefined' && args.registerAction ? args.registerAction : function() {return function() {window.location.reload();};});
	window._payload = payload;

	SporcleLib.Modal._openModalFromAjax('/ajax/modals/verify_social_connect.php', args, function(remodal) {
		SporcleLib.Modal._remodal.$modal.attr({class:'remodal remodal-is-initialized remodal-is-opened verify'});

		remodal.$modal.one('closed', function (e) {
			payload.register_action()();
		});

		if (typeof args != 'undefined' && typeof args.updated != 'undefined' && args.updated) {
			SporcleLib.Modal._remodal.$modal.attr({class:'remodal remodal-is-initialized remodal-is-opened verify success'})
		}
	});
};

SporcleLib.Modal.openMemberOnlyModal = function(args) {
	SporcleLib.Modal._openModalFromAjax('/ajax/modals/memberonly_promo.php', args, function(remodal) {
		remodal.$modal.attr('id', 'memberOnlyPromo');
	});
};

SporcleLib.Modal.openPickModal = function(args, templateVars) {
	SporcleLib.Modal._openModalFromAjax('/ajax/modals/pick_modal.php', args, function(remodal) {
		remodal.$modal.attr('id', 'pickModal');

		var $confirmButton = remodal.$modal.find('#confirm');
		var $cancelButton = remodal.$modal.find('#cancel');
		var $blurb = remodal.$modal.find('#blurb');
		var $charCount = remodal.$modal.find('#char-count');

		$selectDay = remodal.$modal.find("#pick-form select[name='day']");
		$selectMonth = remodal.$modal.find("#pick-form select[name='month']");
		$selectYear = remodal.$modal.find("#pick-form select[name='year']");

		var confirmPick = function() {
			var formValues = remodal.$modal.find('#pick-form').serializeArray();
			var values = {};
			for (var i = 0; i < formValues.length; i++) {
				values[formValues[i].name] = formValues[i].value;
			}

			var data = {
				game_id   : templateVars.game_id,
				pick_date : values['year'] + '-' + values['month'] + '-' + values['day'] + " " + values['time'],
				list_id   : templateVars.category_id ? templateVars.category_id : ($j('#subcat-select').length ? $j('#subcat-select').val() : templateVars.subcategory_id)
			};

			if (typeof values['blurb'] !== 'undefined') {
				data.blurb = values['blurb'];
			}

			jQuery.ajax({
				type        : 'POST',
				url         : '/'+templateVars.pick_type+'s/picks/'+templateVars.pick_type+'picks.php',
				contentType : 'application/json',
				dataType    : 'json',
				data        : JSON.stringify(data)
			})
			.done(function(response) {
				if (templateVars.pick_type == "editor") {
					$j('#makeEditorPickLink').hide();
					$j('#editorPicked').show();
				} else if (templateVars.pick_type == "curator") {
					var months = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
					var scheduledDateDisplay = months[values['month']-1] + " " + values['day'] + ", " + values['year'];
					$j('[data-subcat_id="'+data.list_id+'"]').hide().after($j("<a href='/curators/picks/'>(scheduled "+scheduledDateDisplay+")</a>"));
					window.gameSubcategories[data.list_id]['pick_date'] = response.selecttime;
				}
				remodal.close();
			})
			.fail(function(response) {
				var alert = remodal.$modal.find('.alertBad');

				alert
				.text(response.responseJSON.error)
				.fadeIn();

				setTimeout(function(){alert.fadeOut()}, 2500);
			});
		};

		var populateDaySelect = function() {
			var selectedMonth = parseInt($selectMonth.val());
			var selectedYear = parseInt($selectYear.val());
			var selectedDay = parseInt($selectDay.val());
			var daysInMonth = (new Date(selectedYear, selectedMonth, 0)).getDate();

			$selectDay.empty();
			for (var i = 1; i <= daysInMonth; i++) {
	            $selectDay.append("<option value="+i+" "+(selectedDay == i ? "selected='selected'" : "")+">"+i+"</option>");
	        }
		}
		$selectYear.on('change', function() {
			populateDaySelect();
		});
		$selectMonth.on('change', function() {
			populateDaySelect();
		});

		var formatCharCount = SporcleLib.formatCharCount;
		var maxChars = $blurb.attr("maxlength");
		var startChars = $blurb.val().length;
		$charCount.html(formatCharCount(startChars, maxChars));
	
		$blurb.on('keyup', function(e) {
			var chars = e.target.value.length
			$charCount.html(formatCharCount(chars, maxChars));
		});
		
		$confirmButton.on('click', confirmPick);
		$cancelButton.on('click', function() {
			remodal.close();
		});
	}, templateVars);

	ga('send', 'event', templateVars.pick_type+' Pick Modal', args.analyticAction, window.location.href);
};

SporcleLib.Modal.openFBErrorModal = function() {
	SporcleLib.Modal._openModalFromAjax('/error/fb_connect_oops.php');
};

SporcleLib.Modal.openFBDisconnectModal = function() {
	SporcleLib.Modal._openModalFromAjax('/auth/fb_disconnect_modal.php', {}, function(remodal) {
		remodal.$modal.on('keyup', function(e) {
			if (e.which == 13) {
				disconnectFBUser("/facebook/endpoints/disconnect_fb.php", null);
				remodal.close();
			}
		});

		remodal.$modal.find('.buttonCancel').on('click', function() {
			remodal.close();
		});

		remodal.$modal.find('.buttonYes').on('click', function() {
			disconnectFBUser("/facebook/endpoints/disconnect_fb.php", null);
		});
	});
};

SporcleLib.Modal.openGoogleErrorModal = function() {
	SporcleLib.Modal._openModalFromAjax('/error/google_connect_oops.php');
};

SporcleLib.Modal.openPostChallengeModal = function(args) {
	SporcleLib.Modal._openModalFromAjax('/games/includes/modals/challenge_post.php', args, function(remodal) {
		remodal.$modal.attr('id', 'postChallenge');
	}, null, { ajaxMethod: 'POST' });
};

SporcleLib.Modal.openTriviaLeaguePostGameModal = function(args) {
	SporcleLib.Modal._openModalFromAjax('/games/includes/modals/trivia_league_post.php', args, function(remodal) {
		remodal.$modal.attr('id', 'postLeague');
	}, null, { ajaxMethod: 'POST' });
};

SporcleLib.Modal.openChallengeQuizSelectionModal = function(target, args, callbacks) {
	SporcleLib.Modal._openModalFromAjax('/games/includes/modals/challenge_quiz_select.php', args, function(remodal) {
		remodal.$modal.attr('id', 'challenge-select');

		var gameID;

        if (target && callbacks.close) {
            remodal.$modal.one('closed', function () {
                $j(target).one('click', callbacks.close);
            });
}

		if(remodal.$modal.find('.quiz-data').length){
            var $icon = remodal.$modal.find('.quiz-data .icon');
            var $name = remodal.$modal.find('.quiz-data .name');
            var $creator = remodal.$modal.find('.quiz-data .creator');
            var $category = remodal.$modal.find('.quiz-data .category');
            var $gameType = remodal.$modal.find('.quiz-data .gameType');
            var $time = remodal.$modal.find('.quiz-data .time');

            function getRandomQuizData() {
                $j.ajax({
                    url  : '/ajax/challenge/random_quiz.php',
                    type : 'get',
                    data : {cid: args.cid}
                }).done(function(data) {
                    $icon.css({'background-image': 'url('+data.game_image+')'});
                    $name.html(data.game_name);
                    $creator.html('by '+data.creator_handle);
                    $category.html(data.category_name);
                    $gameType.removeClass().addClass('gameType '+data.game_type_underscore).attr({title: data.game_type_description});
                    $time.html(data.game_timer);
                    gameID = data.encoded_game_id;
                }).fail(function(xhr) {
                    remodal.$modal.find('.quiz-data').html('<div class="alertBad" style="flex:1"><span class="alert-message">Oops, something went wrong. Please refresh and try again.</span></div>');
                    remodal.$modal.find('#shuffle-quiz-button, #select-quiz-button').addClass('disabled').attr({disabled: true}).off('click');
                });
		}

            remodal.$modal.find('#shuffle-quiz-button').on('click', getRandomQuizData);

            remodal.$modal.find('#select-quiz-button').one('click', function() {
                SporcleLib.Modal.openChallengeModal(target, {gid : gameID, sc: 0, switching: 1}, callbacks);
			});

            getRandomQuizData();
		}
	});
};

SporcleLib.Modal.openChallengeModal = function(target, args, callbacks) {
	SporcleLib.Modal._openModalFromAjax('/games/includes/modals/game_page_challenge.php', args, function(remodal) {
		remodal.$modal.attr('id', 'preChallenge');

		var checkAllCheckbox    = remodal.$modal.find('#check_all_checkbox'),
			challengeMessageBox = remodal.$modal.find('#chl-message-box'),
			allowMultiSelect    = remodal.$modal.find('#select_all_friends').length;

		function areAllVisibleFriendsSelected() {
			return remodal.$modal.find('.challenge-friend.clicked:visible').length === remodal.$modal.find('.challenge-friend:visible').length;
		}

		function updateSelectAllCheckbox(checked){
			if (typeof checked === 'undefined') {
				checked = areAllVisibleFriendsSelected();
		}
			checkAllCheckbox.prop('checked', checked);
		}

		function toggleSelectAllCheckbox() {
			var makeSelected = !areAllVisibleFriendsSelected();

			remodal.$modal.find('.challenge-friend:visible').each(function() {
				if (!makeSelected) {
					$j(this).removeClass('clicked').find('.friend_input').prop('checked', makeSelected);
				} else {
					$j(this).addClass('clicked').find('.friend_input').prop('checked', makeSelected);
				}
			});

			updateSelectAllCheckbox(makeSelected);
		}

		function filterChallengeFriends() {
			var text = $j('#search-friend').val().toLowerCase();
			var playType = $j('#friends-play-type')[0] ? $j('#friends-play-type').val() : 'all';

			$j('.challenge-friend').each(function() {
				var friend = $j(this);

				if (
					((playType === 'all') || (playType === 'unplayed' && !friend.data('lastplayed')))
					&& (text === '' || friend.data('name').toString().indexOf(text) >= 0)
				) {
					friend.show();
				} else {
					friend.hide();
				}
			});

			updateSelectAllCheckbox();
		}

        function sendChallenge() {
			var recipients = [];

			// Set up recipients
			$j('.friend_input').each(function(i, val) {
				var e = $j(val);
				if (e.prop('checked') && !e.prop('disabled')) {
					recipients.push(e.val());
				}
			});

			// Make sure that some recipients have been checked
			if (recipients.length === 0) {
				var msg = allowMultiSelect ? 'Please select at least one friend' : 'Please select a friend';
				remodal.$modal.find('#chl-message').html(msg);
				challengeMessageBox.removeClass().addClass('alertBad').fadeIn(100);

				// Rebind click handler
				remodal.$modal.find('#challenge-send').one("click", function() {
					sendChallenge();
				});
				return;
			}

			// Send Challenges
				$j('#challenge-send').hide();
				$j('#challenge-ajax-loader').show();

				// Need to stringify opponent_ids so we don't blow the POST variable limit
				$j.ajax({
					url: '/challenge/ajax/new.php',
					type: 'post',
					data: {
						game_id      : args.gid,
					opponent_ids : JSON.stringify(recipients),
					},
					dataType: 'json'
				})
				.always(callbacks.always)
				.done(callbacks.done)
				.fail(callbacks.fail);
			}

		callbacks = $j.extend({}, {
			always: function() {
				remodal.close();
			},
			done: function() {},
			fail: function() {},
		}, callbacks);


		// ----------------------------------------
		// Event Handlers
		// ----------------------------------------
		// Filter the list of friends with the dropdown & search box
		remodal.$modal.find('#friends-play-type').on('change', filterChallengeFriends);
		remodal.$modal.find('#search-friend').on('keyup', filterChallengeFriends);

		// Set the "select all" container to check the checkbox regardless of where in the container you click
		remodal.$modal.find('#select_all_friends').on('click', toggleSelectAllCheckbox);

		// Set the friend row to check the checkbox regardless of where in the row you click
		remodal.$modal.find('.challenge-friend').on('click', function(e) {
			if (challengeMessageBox.is(':visible')) {
				challengeMessageBox.fadeOut(100);
			}

			var inputType = allowMultiSelect ? 'checkbox' : 'radio';

			var $this = $j(this);

			// If they didn't click the actual checkbox, this will set it to be checked.
			if (e.target.type !== inputType) {
				var input = $this.find('.friend_input');
				if (!input[0].disabled) {
					input.prop('checked', function(i,val) { return !val; })
				}
			}

			if (allowMultiSelect) {
				$this.toggleClass('clicked');
			} else {
				remodal.$modal.find('.challenge-friend').not($this).removeClass('clicked');
				$this.addClass('clicked');
			}

			updateSelectAllCheckbox();
		});

		// Send the challenge, bind on .one not .click to prevent duplicate challenges going out
		remodal.$modal.find('#challenge-send').one("click", sendChallenge);

		// Close the alert message in the challenge modal
		remodal.$modal.find('#chl-message-close').on('click', function() {
			remodal.$modal.find('#chl-message-box').fadeOut(100);
		});

		if (target && callbacks.close) {
			remodal.$modal.one('closed', function () {
				$j(target).one('click', callbacks.close);
	});
}

	});
};

SporcleLib.Modal.openReportGameModal = function(args) {
	SporcleLib.Modal._openModalFromAjax('/games/includes/modals/report_'+args.form+'.php', { game_id : gameID }, function(remodal) {
		remodal.$modal.attr('id', 'reportGame');
		var $reportMessage = remodal.$modal.find('#report_message');

		remodal.$modal.on('opened', function() {
			$reportMessage.trigger('focus');
		});

		remodal.$modal.find('#sendReport').on('click', _.once(function() {
			var gameMessage = $j('#game-message');
			var gameMessageBox = $j('#game-message-box');


			var reportMessage = $reportMessage.val();

			if ((args.form == 'published') && (reportMessage == "")) {
				gameMessage.html('Please provide us with some details in your report.');
				gameMessageBox.addClass('alertBad');
			}

			var params = 'game_id='+gameID+'&message='+encodeURIComponent(reportMessage);
			params += '&reason='+encodeURIComponent($j('#report_reason').val());

			$j.ajax({
				url: '/games/ajax/report.php',
				type: 'post',
				data: params,
			}).done(function(xhr) {
				gameMessage.html('Game reported! Thank you for alerting us to this quiz.');
				gameMessageBox.addClass("alertGood");
				gameMessageBox.fadeIn(100);
			}).fail(function(xhr) {
				gameMessage.html("Unable to submit report. Please try again later.");
				gameMessageBox.addClass("alertBad");
				gameMessageBox.fadeIn(100);
			})
			.always(function() {
				remodal.close();
			});
		}))
	});
};

SporcleLib.Modal.openReportedModal = function(args) {
	SporcleLib.Modal._openModalFromAjax('/games/includes/modals/reported.php', {
        game_id: args.game_id
    }, function(remodal) {

        var $dismiss = remodal.$modal.find('.dismiss-report');
        $dismiss.on('click', function(e) {
            var elID = $j(e.currentTarget).attr('id');
            elID = elID.replace("dismiss-", "");
            var type = elID.includes('review') ? 'review' : 'report';
            var id = elID.replace(type+'-', '');

            var handleComment = $j('#comment-'+type+'-'+id); //content
            var reporterID = $j('#reporter-id-'+type+'-'+id); //value
            var notify = $j('#notify-reporter-'+type+'-'+id); //value

            $j.ajax({
                url: '/games/ajax/handle_reports.php',
                type: 'post',
                data: {
                    'action'      : 'dismiss',
                    'game_id'     : window.Sporcle.gameData.gameID,
                    'type'        : type,
                    'id'          : id,
                    'reporter_id' : $j(reporterID).val(),
                    'notify'      : $j(notify).val()
                },
            }).done(function(response) {
                remodal.$modal.find('.alertBad').hide();
                $j('#quiz-report-'+type+'-'+id).remove();

                if ($j('.quiz-report').length == 0) {
                    $j('#reported-link').hide();
                    remodal.close();
                }
            }).fail(function(response) {
                remodal.$modal.find('.alertBad').text(response.responseJSON.error);
                remodal.$modal.find('.alertBad').show();
            });
        });

        var $handle = remodal.$modal.find('.handle-report');
        $handle.on('click', function(e) {
            var elID = $j(e.currentTarget).attr('id');
            elID = elID.replace("handle-comment-", "");
            var type = elID.includes('review') ? 'review' : 'report';
            var id = elID.replace(type+'-', '');

            var handleSection = $j('#comment-section-'+type+'-'+id);

            handleSection.show();
        });

        var $saveComment = remodal.$modal.find('.save-report-comment');
        $saveComment.on('click', function(e) {
            var elID = $j(e.currentTarget).attr('id');
            elID = elID.replace("save-comment-", "");
            var type = elID.includes('review') ? 'review' : 'report';
            var id = elID.replace(type+'-', '');

            var handleComment = $j('#comment-'+type+'-'+id); //content
            var reporterID = $j('#reporter-id-'+type+'-'+id); //value
            var notify = $j('#notify-reporter-'+type+'-'+id); //value

            $j.ajax({
                url: '/games/ajax/handle_reports.php',
                type: 'post',
                data: {
                    'action'      : 'handle',
                    'game_id'     : window.Sporcle.gameData.gameID,
                    'type'        : type,
                    'comment'     : handleComment.val(),
                    'id'          : id,
                    'reporter_id' : reporterID.val(),
                    'notify'      : notify.val()
                },
            }).done(function(response) {
                remodal.$modal.find('.alertBad').hide();
                $j('#quiz-report-'+type+'-'+id).remove();

                if ($j('.quiz-report').length == 0) {
                    $j('#reported-link').hide();
                    remodal.close();
}
            }).fail(function(response) {
                remodal.$modal.find('.alertBad').text(response.responseJSON.error);
                remodal.$modal.find('.alertBad').show();
            });
        });
	});
};

SporcleLib.Modal.openReportCommentModal = function(args) {
	SporcleLib.Modal._openModalFromAjax('/games/includes/modals/report_comment.php', {
    }, function(remodal) {
		remodal.$modal.attr('id', 'confirmReport'+args.comment_id);

		remodal.$modal.find('#confirmReport').on('click', _.once(function() {
			args.callback();
            remodal.close();
		}))

        remodal.$modal.find('#cancelReport').on('click', _.once(function() {
			remodal.close();
		}))
	});
}

SporcleLib.Modal.openEmbedGameModal = function(args) {
	SporcleLib.Modal._openModalFromAjax('/games/includes/modals/embed.php', {
        game: args.game_id
    }, function(remodal) {
		remodal.$modal.attr('id', 'embedGame');

		var typingTimer;
		var width;

		var getEmbedCode = function() {
			var newWidth;

			var selected = remodal.$modal.find('#embed-size-select option:selected').val();

			if (selected === "-") {
				// Custom size selected
				remodal.$modal.find('#custom-size').show();

				newWidth = parseInt(remodal.$modal.find('#custom-size-input').val(), 10);
				if (width === newWidth) {
					return;
				}
				width = newWidth;

			} else {
				remodal.$modal.find('#custom-size').hide();

				newWidth = parseInt(selected, 10);
				if (width === newWidth) {
					return;
				}
				width = newWidth;
			}

			if (isNaN(width)) {
				remodal.$modal.find('#embed-code').removeClass().empty();
				return;

			}

			// Get the embed code
			$j.ajax({
				url: '/embed/gencode.php',
				type: 'post',
				data: {g: args.game_id, w: width}

			}).done(function(data) {
				remodal.$modal.find('#embed-code').removeClass().html(data);

			}).fail(function(xhr) {
				var error = xhr.responseText ? xhr.responseText : "Oops, something went wrong. Please refresh and try again.";
				remodal.$modal.find('#embed-code').addClass('error').html(error);
			});
		};

		// Set up handlers
		remodal.$modal.find('#embed-size-select').on('change', getEmbedCode);
		remodal.$modal.find('#custom-size-input').on('keyup', function() {
			clearTimeout(typingTimer);
			typingTimer = setTimeout(getEmbedCode, 300);
		});

		// Trigger inital fetch of the embed code
		$j('#embed-size-select').trigger('change');
	});
};

SporcleLib.Modal.openPauseModal = function(args) {
	SporcleLib.Modal._openModalFromAjax('/games/includes/pause.php', args, function(remodal) {
		remodal.settings.closeOnOutsideClick = false;
		remodal.$overlay.addClass('pauseScreen');
		remodal.$modal.addClass('pauseScreen');

		setTimeout(function(){remodal.$modal.find('#resumeBtn').show();},800);

		remodal.$modal.find('#resumeBtn').on('click', function() {

            var pausepagePlaybuzz = document.getElementById('stream-sdk-jssdk_sporcle.com_Test');
            if(pausepagePlaybuzz){
                pausepagePlaybuzz.parentNode.removeChild(pausepagePlaybuzz);
            }
			remodal.close();
		});

        var gamepagePlaybuzz = $j('#playbuzz-gamepage-anchor');

        if(gamepagePlaybuzz.length){
            gamepagePlaybuzz.css('visibility', 'hidden');
        }

		remodal.$modal.one('closed', function () {
			remodal.settings.closeOnOutsideClick = true;
			remodal.$overlay.removeClass('pauseScreen');
			remodal.$modal.removeClass('pauseScreen');

            if(gamepagePlaybuzz.length){
                gamepagePlaybuzz.css('visibility', 'visible');
            }

			unPauseGame();
			remodal.$modal.off('closed');
		});
	});
};

SporcleLib.Modal.openBlurPunishModal = function(args) {
	SporcleLib.Modal._openModalFromAjax('/games/includes/showdown_blur_modal.php', args, function(remodal) {
		remodal.settings.closeOnOutsideClick = false;
		remodal.$overlay.addClass('pauseScreen');
		remodal.$modal.addClass('pauseScreen');

		var punishTime = 30;
		switch (args.blur_count) {
			case 1: punishTime = 0; break;
			case 2: punishTime = 5; break;
			case 3: punishTime = 15; break;
			default: break;
		}

		var countdownTimer = punishTime;
		if (countdownTimer > 0) {
			punishTime = punishTime * 1000;
			remodal.$modal.find('#countdown').text(countdownTimer);
			remodal.$modal.find('#dynamicMessage').text("Continue in");

			var interval = setInterval(function() {
				countdownTimer--;
				if (countdownTimer == 0) {
					remodal.$modal.find('#showdownResume').removeClass('disabled');
					remodal.$modal.find('#countdown').text("");
					remodal.$modal.find('#dynamicMessage').text("Continue your Showdown with " + args.opponent_name);
					clearInterval(interval);
				} else {
					remodal.$modal.find('#countdown').text(countdownTimer);
				}

			}, 1000);
		} else {
			remodal.$modal.find('#showdownResume').removeClass('disabled');
			remodal.$modal.find('#dynamicMessage').text("Continue your Showdown with " + args.opponent_name);
		}

		remodal.$modal.find('#showdownResume').on('click', function() {
			window.showdown.vm.api.closeBlurPunishModal();
			remodal.close();
		});

		remodal.$modal.one('closed', function () {
			remodal.settings.closeOnOutsideClick = true;
			remodal.$overlay.removeClass('pauseScreen');
			remodal.$modal.removeClass('pauseScreen');
			remodal.$modal.off('closed');
		});
	});
};

SporcleLib.Modal.openEditTagsModal = function(args) {
	SporcleLib.Modal._openModalFromAjax('/games/includes/modals/edit_tags.php', args, function(remodal) {
		remodal.$modal.find('#save-tags').on('click', function() {
			remodal.close();
		});
		remodal.$modal.find('.tagactive').on('click', detag);
	});
};

SporcleLib.Modal.openBingoFeedbackModal = function(args) {
	SporcleLib.Modal._openModalFromAjax('/games/includes/modals/feedback_bingo.php', args, function(remodal) {
		remodal.settings.closeOnOutsideClick = false;

		remodal.$modal.find('#message').on('keyup', function() {
			remodal.$modal.find('#feedback-required').hide();
		});

		remodal.$modal.find('#email-address').on('keyup', function() {
			remodal.$modal.find('#email-required').hide();
			remodal.$modal.find('#email-invalid').hide();
		});

		remodal.$modal.find('#feedbackButton').on('click', function() {
			var rating = remodal.$modal.find('input:radio[name=rating]:checked').val();

			var feedback = remodal.$modal.find('#message').val();
			if (!feedback) {
				remodal.$modal.find('#feedback-required').show();
			}

			var email = remodal.$modal.find('#email-address').val();
			if (!email) {
				remodal.$modal.find('#email-required').show();
				email = null;
			} else if (!(/^[-a-z0-9!#$%&\'*+\/\=?^_`{|}~\.]+@[a-z0-9.-]+\.[a-z]{2,4}$/i.test(email))) {
				remodal.$modal.find('#email-invalid').show();
				email = null;
			}

			if (feedback && email) {
				$j.ajax({
					url: "bingo_feedback.php",
					type: "POST",
						data: {rating: rating, message: encodeURIComponent(feedback), email: encodeURIComponent(email)},
						dataType: 'html'
				}).done(function(data) {
					remodal.$modal.find('#modalcontent').html(data);
				});
			}
		});

		remodal.$modal.on('opened', function() {
			remodal.$modal.find('#message').trigger('focus');
		});
	});
};

SporcleLib.Modal.openReportLive5Modal = function(args) {
	SporcleLib.Modal._openModalFromAjax('/games/includes/modals/report_livefive.php', args, function(remodal) {
		remodal.$modal.attr('id', 'report5');

		remodal.$modal.on('opened', function() {
			remodal.$modal.find('#message').trigger('focus');
		});

		remodal.$modal.find('#sendReport').on('click', function() {
			$j.ajax({
				url      : "/livefive/livefive_report.php",
				type     : "POST",
				data     : {
					question_id : args.question_id,
					reason      : remodal.$modal.find('#report_reason').val(),
					message     : encodeURIComponent(remodal.$modal.find('#message').val())
				},
				dataType : 'html'
			}).done(function(data) {
				remodal.$modal.find('.reportContent').html(data);
			});
		});
	});
};

SporcleLib.Modal.openUpdateReasonModal = function(args) {
	SporcleLib.Modal._openModalFromAjax('/create/gamestatechange.php', {}, function(remodal) {
		remodal.$modal.attr('id', 'updateReason');

		resetdrop = args.dropdown;

		remodal.$modal.find('#reason_submit').on('click', function(e) {
			$j(this).prop('disabled', true); // stops possibility of multiple submits
			addInternalNote();
			$j('#internalnote').val($j('#internalnote').val() + ($j('#internalnote').val().length > 0 ? ' ' : '') + $j('#reason_message').val());
			resetdrop = null;
			remodal.close();
		});

		remodal.$modal.on('closed', function() {
			if(resetdrop) {
				resetdrop.selectedIndex = selecteddrop;
				resetdrop = null;
			}
		});

		setTimeout(function() {
			remodal.$modal.find('#reason_message').trigger('focus');
		});
	});
};

SporcleLib.Modal.openDashboardCSVModal = function() {
    SporcleLib.Modal._openModalFromAjax('/create/modals/dashboard_csv.php', {}, function(remodal) {
        var error = $j("#error");
        error.text('');
        remodal.$modal.find('#download-csv').on('click', function() {
            var tab = $j("#dashboard-csv-tab").val();
            var spinner = $j("#ajax-loader");

            var self = $j(this);

            self.hide();
            spinner.show();

            $j.ajax({
                type: 'GET',
                url: '/create/ajax/dashboard_data_csv.php',
                data: {tab: tab},
                success: function(response){

                    if(typeof response === 'object' && response.hasOwnProperty('success')){

                        if(!response.success && response.error){
                            error.text(response.error);
                        } else if (!response.success){
                            error.text("Oops! Something went wrong. Please refresh and try again.");
                        }

                        spinner.hide();
                        self.show();

                    } else {
                        var today = new Date();
                        var date = today.getFullYear()+'-'+(today.getMonth()+1)+'-'+today.getDate();
                        var time = today.getHours() + "-" + today.getMinutes() + "-" + today.getSeconds();
                        var dateTime = date+'_'+time;
                        var file = tab + '_data_' + dateTime + '.csv';
                        downloadDashboardCSV(file, response);
                        remodal.close();
                    }
                },
                error: function (err) {
                    error.text("Oops! Something went wrong. Please refresh and try again.");
                    spinner.hide();
                    self.show();
                }
            })
        });
    });
};

function downloadDashboardCSV(filename, data){
    var blob = new Blob([data], { type: 'text/csv;charset=utf-8;' });
    if (navigator.msSaveBlob) {
        navigator.msSaveBlob(blob, filename);
    } else {
        var link = document.createElement("a");
        if (link.download !== undefined) {
            var url = URL.createObjectURL(blob);
            link.setAttribute("href", url);
            link.setAttribute("download", filename);
            link.style.visibility = 'hidden';
            document.body.appendChild(link);
            link.click();
            document.body.removeChild(link);
        }
    }
}

function reorderTabIndex(startingIndex)
{
	startingIndex = parseInt(startingIndex, 10);
	var allElements = [];
	allElements = document.body.getElementsByTagName("*");
	var allElementsLength = allElements.length;

	for (var i = 0; i < allElementsLength; i++)
	{
		if (allElements[i].getAttribute("tabindex") != null)
		{
			var currentIndex = allElements[i].getAttribute("tabindex");
			currentIndex = parseInt(currentIndex, 10);
			allElements[i].setAttribute("tabindex", startingIndex + currentIndex);
		}
	}
}

function validateEmail(email) {
    var re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    return re.test(String(email).toLowerCase());
}

SporcleLib.Modal.openCommonConnectionsModal = function(args) {
    SporcleLib.Modal._openModalFromAjax('/find-friends/ajax/common_connections_modal.php', args, function(remodal) {
        remodal.$modal.attr('id', 'commonConnectionsModal');
    });
};

SporcleLib.Modal.openReportVenueModal = function(args) {
    SporcleLib.Modal._openModalFromAjax('/events/includes/report_venue_modal.php', {}, function(remodal) {
        var ajaxLoader = '<div id="report-ajax-loader"></div>';
        var emailInput = remodal.$modal.find('#email');
        var extraInfoInput = remodal.$modal.find('#extraInfo');
        var sendReportButton = remodal.$modal.find("#sendReport");
        var inputErrors = remodal.$modal.find(".error");
        var genError = remodal.$modal.find("#general-error");
        inputErrors.text('');
        genError.text('');

        remodal.$modal.find('#sendReport').on('click', function() {
            emailInput.css('border', '');
            extraInfoInput.css('border', '');

            var email = emailInput.val().trim();
            var reason = remodal.$modal.find('#reason').val().trim();
            var extraInfo = extraInfoInput.val().trim();
            var hasErrors = false;
            genError.text('');
            inputErrors.text('');

            if(!validateEmail(email)){
                remodal.$modal.find('.email-error').text('Incorrect email format');
                remodal.$modal.find('#email').css('border', '2px solid red');
                hasErrors = true;
            }

            if(!email){
                remodal.$modal.find('.email-error').text('Required');
                remodal.$modal.find('#email').css('border', '2px solid red');
                hasErrors = true;
            }

            if(!extraInfo){
                remodal.$modal.find('.extraInfo-error').text('Required');
                remodal.$modal.find('#extraInfo').css('border', '2px solid red');
                hasErrors = true;
            }

            if(hasErrors) return;

            sendReportButton.html(ajaxLoader);

            $j.ajax({
                url: "/ajax/events/report_venue.php",
                type: "POST",
                data: {
                    venueID: args.venueID,
                    email: email,
                    reason: reason,
                    extraInfo: extraInfo
                }
            }).done(function(data) {
                emailInput.css('border', '');
                extraInfoInput.css('border', '');

                $j('#report-venue-link, #report-venue-atag')
                    .css({
                    cursor: 'default',
                    textDecoration: 'none'
                })
                    .removeAttr('title')
                    .removeClass('tooltip')
                    .off();

                remodal.close();

                showMainPageAlert('Thanks for your report! We will investigate the issue.', {type: 'success'});

            }).fail(function (err) {

                inputErrors.text('');
                emailInput.css('border', '');
                extraInfoInput.css('border', '');
                genError.text('Oops! Something went wrong. Please refresh and try again.');
                sendReportButton.html('Send Report');

            })
        });
    });
};

SporcleLib.Modal.openPreviouslyPlayedSetModal = function(args) {
	const html = $j('#previously-played-set-template').html();
	
	const callback = function(remodal) {
		remodal.$modal.find('#btn-yes').on('click', function() {
			remodal.close();
			args.onYes();
		})
		
		remodal.$modal.find('#btn-no').on('click', function() {
			remodal.close();
		})
	}
	
	const templateVars = {
		date : args.date
	}
	
	SporcleLib.Modal._openModal(html, {}, callback, templateVars);
}

SporcleLib.Modal.openConfirmationModal = function(args) {
    var html = $j('#sporcle-confirmation-modal-template').html();

    var callback = function(remodal) {
        remodal.$modal.attr('id', 'confirmationModal');

        remodal.$modal.find('.confirmation-button #yes').on('click', function() {
            if(args.hasOwnProperty('onYes')){
                args.onYes(remodal);
            } else {
                console.log("onYes() not implemented");
                showMainPageAlert('Oops! Something went wrong. Please contact <a href="/feedback/">feedback</a> with as much detail as possible.')
            }
        });

        remodal.$modal.find('.confirmation-button #no').on('click', function() {
            if(args.hasOwnProperty('onNo')){
                args.onNo(remodal);
            }

            remodal.close();
        });


        remodal.$modal.on('keydown', function (e) {
            if(e.which == 13 && remodal.$modal.hasClass('remodal-is-opened')){
                if(args.hasOwnProperty('onYes')){
                    remodal.$modal.off('keydown');
                    args.onYes(remodal);
                }
            } else if (e.which == 27){
                remodal.close();
                e.stopImmediatePropagation(); //in case other JS on the page attached an escape keydown listener to the document (i.e. grid)
            }
        });
    };

    var templateVars = {
        msg: args.hasOwnProperty('msg') ? args.msg : 'Are you sure you want to do this?',
        yt: args.hasOwnProperty('yt') ? args.yt : 'Yes',
        nt: args.hasOwnProperty('nt') ? args.nt : 'No'
    };

    var remodalArgs = {
        onClose: function (remodal) {
            remodal.$modal.off('keydown');

            if(args.hasOwnProperty('always')){
                args.always();
            }
        }
    };

    SporcleLib.Modal._openModal(html, remodalArgs, callback, templateVars);
};

SporcleLib.Modal.openVirtualTriviaVideoModal = function() {
	var html = $j('#video-modal-template').html();
	SporcleLib.Modal._openModal(html, {}, function(remodal) {
		remodal.$modal.one('closed', function (e) {
			remodal.$modal.empty();
		});
	});
};

SporcleLib.Modal.openQuestObjectiveModal = function(objectiveID, callback) {
	SporcleLib.Modal._openModalFromAjax('/quests/ajax/objectivemodal.php', {
		action       : 'get_modal',
		objective_id : objectiveID,
	}, function(remodal) {
		SporcleLib.Modal._remodal.$modal.attr({class:'remodal remodal-is-initialized remodal-is-opened quest'});
		
		callback();
	});
};

// Traps the focus to within the current modal
function trapModalFocus() {
	// Find the tabbable elements
	SporcleLib.Modal.tabbableList = $j('.remodal-wrapper .remodal-is-opened')
                .find('select, input, textarea, button, a')
                .filter(':visible');

	const firstTabbable = SporcleLib.Modal.tabbableList.first();
	const lastTabbable = SporcleLib.Modal.tabbableList.last();

	// IF they they press Tab on the last element in modal the focus moves to the first element in modal.
	lastTabbable.on('keydown', function (e) {
		if (e.which === 9 && !e.shiftKey) {
			e.preventDefault();
			firstTabbable.focus();
		}
	});

	// Vice versa to event listener above when the user presses Shift + Tab.
	firstTabbable.on('keydown', function (e) {
		if (e.which === 9 && e.shiftKey) {
			e.preventDefault();
			lastTabbable.focus();
		}
	});
}

function returnFocusFromModal() {
	$j(SporcleLib.Modal.lastClickedEl).focus();

	SporcleLib.Modal.lastClickedEl = undefined;
}